/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must 
 *    not be used to endorse or promote products derived from this software 
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without 
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project (http://java.apache.org/)."
 *    
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
 
package org.apache.jmeter.visualizers;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * This class implements a statistical analyser that 
 * calculates both the average and the standard deviation
 * of the sampling process and outputs them as autoscaling
 * plots.
 *
 * @author  <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 * @version $Revision: 1.1 $ $Date: 1999/02/27 14:05:39 $
 */
public class GraphVisualizer extends JPanel implements Visualizer, ItemListener {

    public class Sample {
        
        public long data;
        public long average;
        public long deviation;
        
        public Sample(long data, long average, long deviation) {
            this.data = data;
            this.average = average;
            this.deviation = deviation;
        }
    }

    public class Graph extends JComponent implements Scrollable {

        private boolean data = true;
        private boolean average = true;
        private boolean deviation = true;
        
        private long averageSum = 0;
        private long variationSum = 0;
        private long counter = 0;
        private long previous = 0;
        private long max = 1;
        
        private Vector samples;
        
        public Graph() {
            this.samples = new Vector(500);
            this.setPreferredSize(new Dimension(2000, 100));
        }

        public Sample add(long sample) {
            counter++;
            if (sample > max) max = sample;
            averageSum += sample;
            long average = averageSum / counter;
            variationSum += Math.pow(sample - average, 2);
            long deviation = (long) Math.pow(variationSum / counter, 0.5);
            Sample s = new Sample(sample, average, deviation);
            previous = sample;
            samples.addElement(s);
            
            return s;
        }
            
        public void clear() {
            samples.removeAllElements();
            averageSum = 0;
            variationSum = 0;
            counter = 0;
            previous = 0;
            max = 1;
        }

        public void enableData(boolean value) {
            this.data = value;
        }

        public void enableAverage(boolean value) {
            this.average = value;
        }
        
        public void enableDeviation(boolean value) {
            this.deviation = value;
        }
        
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            
            Dimension d = getSize();
            
            Enumeration e = this.samples.elements();
            for (int i = 0; e.hasMoreElements(); i++) {
                Sample s = (Sample) e.nextElement();
                
                if (data) {
                    int data = (int) (s.data * d.height / max);
                    g.setColor(Color.black);
                    g.fillRect(i, d.height - data, 1, 2);
                }
                
                if (average) {
                    int average = (int) (s.average * d.height / max);
                    g.setColor(Color.blue);
                    g.fillRect(i, d.height - average, 1, 2);
                }
                
                if (deviation) {
                    int deviation = (int) (s.deviation * d.height / max);
                    g.setColor(Color.red);
                    g.fillRect(i, d.height - deviation, 1, 2);
                }
            }
        }
        
        public Dimension getPreferredScrollableViewportSize() {
            return new Dimension(400, 100);
        }

        public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
            return 1;    
        }

        public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
            return 50;
        }

        public boolean getScrollableTracksViewportWidth() {
            return false;
        }

        public boolean getScrollableTracksViewportHeight() {
            return true;
        }
    }
        
    private Graph graph;
    private JCheckBox data;
    private JCheckBox average;
    private JCheckBox deviation;
    private JTextField dataField;
    private JTextField averageField;
    private JTextField deviationField;

    public GraphVisualizer() {
        super();
        
        data = new JCheckBox("data");
        data.setSelected(true);
        data.addItemListener(this);
        data.setForeground(Color.black);
        average = new JCheckBox("average");
        average.setSelected(true);
        average.addItemListener(this);
        average.setForeground(Color.blue);
        deviation = new JCheckBox("deviation");
        deviation.setSelected(true);
        deviation.addItemListener(this);
        deviation.setForeground(Color.red);

        JPanel checkPanel = new JPanel();
        checkPanel.setLayout(new GridLayout(0, 1));
        checkPanel.add(data);
        checkPanel.add(average);
        checkPanel.add(deviation);
        
        graph = new Graph();
        JScrollPane graphScrollPanel = new JScrollPane(graph);
        graphScrollPanel.setViewportBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
        //graphScrollPanel.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        //graphScrollPanel.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        
        dataField = new JTextField("0000");
        dataField.setEditable(false);
        dataField.setForeground(Color.black);
        dataField.setBackground(getBackground());
        averageField = new JTextField("0000");
        averageField.setEditable(false);
        averageField.setForeground(Color.blue);
        averageField.setBackground(getBackground());
        deviationField = new JTextField("0000");
        deviationField.setEditable(false);
        deviationField.setForeground(Color.red);
        deviationField.setBackground(getBackground());
        
        JPanel showPanel = new JPanel();
        showPanel.setLayout(new GridLayout(0, 1));
        showPanel.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));
        showPanel.add(dataField);
        showPanel.add(averageField);
        showPanel.add(deviationField);

        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        setLayout(new BorderLayout());
        add(checkPanel, BorderLayout.WEST);
        add(graphScrollPanel, BorderLayout.CENTER);
        add(showPanel, BorderLayout.EAST);
    }    

    public void itemStateChanged(ItemEvent e) {
        if (e.getItem() == data) {
            this.graph.enableData(e.getStateChange() == ItemEvent.SELECTED);
        } else if (e.getItem() == average) {
            this.graph.enableAverage(e.getStateChange() == ItemEvent.SELECTED);
        } else if (e.getItem() == deviation) {
            this.graph.enableDeviation(e.getStateChange() == ItemEvent.SELECTED);
        }
        this.graph.repaint();
    }        

    public synchronized void add(long sample) {
        Sample s = this.graph.add(sample);

        dataField.setText(Long.toString(s.data));
        averageField.setText(Long.toString(s.average));
        deviationField.setText(Long.toString(s.deviation));
        
        repaint();
    }
    
    public synchronized void clear() {
        this.graph.clear();
        
        dataField.setText("0000");
        averageField.setText("0000");
        deviationField.setText("0000");
        
        repaint();
    }
    
    public String toString() {
        return "Show the samples analysys as dot plots";
    }
}